const fs = require('fs');
const exec = require('child_process').exec;

const gulp = require('gulp');
const plugins = require('gulp-load-plugins')();
const browserSync = require('browser-sync').create();
const proxy = require('http-proxy-middleware');

const api = proxy('/api', {
  target: 'https://translate.google.cn/',
  changeOrigin: true,
  pathRewrite: {
    '^/api': ''
  },
  logLevel: 'debug'
});

const utils = require('./config/utils');
const {
  produce,
  defaultEnv,
  sourceMap,
  produceFn,
  rf
} = require('./config/task/if-fn');

const {
  scssOptions,
  esOptions,
  syncs,
  getPath
} = require('./config/gulp');

//  task 命名规则
//  watch => watch + name
//  任务 => gulp + name

/// 编译 Scss 及发布 CSS
gulp.task('gulp-sass', () => {
  console.log('开始编译 Scss 文件');
  return gulp
    .src(scssOptions.entry)
    .pipe(plugins.if(
      sourceMap, 
      plugins.sourcemaps.init()
    ))
    .pipe(plugins
      .sass(
        { 
          outputStyle: 'expanded' 
        }
      )
      .on('error', plugins.sass.logError)
    )
    .pipe(plugins.if(
      produceFn, 
      plugins.cleanCss())
    )
    .pipe(plugins.concat(scssOptions.name))
    .pipe(plugins.if(
      sourceMap, 
      plugins.sourcemaps.write(scssOptions.sourcemap)
    ))
    .pipe(gulp.dest(scssOptions.output));
});

/// 编译 ES 及发布 JS
gulp.task('gulp-es', () => {
  console.log('开始编译 Javascript 文件');
  return gulp
    .src(esOptions.entry)
    .pipe(plugins.plumber(
      {
          errorHandler: plugins.notify.onError('ES6 Error: <%= error.message %>')
      }
    ))
    .pipe(plugins.if(
      sourceMap, 
      plugins.sourcemaps.init()
    ))
    .pipe(plugins.babel(
      { 
        presets: [
          // 'es2015', 
          // 'stage-0',
          'stage-1',
          // 'stage-2',
          // 'stage-3',
        ],
        plugins: [
          //让jsx支持 if/else 表达式
          'transform-do-expressions',
          'transform-vue-jsx'
        ]
      }
    ))
    .pipe(plugins.if(
      produceFn, 
      plugins.uglify()
    ))
    .pipe(plugins.if(
      sourceMap, 
      plugins.sourcemaps.write(esOptions.sourcemap)
    ))
    .pipe(gulp.dest(esOptions.output));
});

/// 监听 scss 文件
gulp.task('watch-sass', () => {
  return gulp
    .watch(scssOptions.files, ['gulp-sass'])
    .on('change', () => {
      utils.printDate('发现更改了 Scss 文件');
    })
});

/// 监听 js 文件
gulp.task('watch-es', () => {
  return gulp
    .watch(esOptions.files, ['gulp-es'])
    .on('change', () => {
      utils.printDate('发现更改了 Javascript 文件');
    })
});

/// 清除发布目录下的发布的 js、css 文件
gulp.task('clean-jscss', () => {
  console.log('清理编译文件');
  return gulp
    .src(
      [
        `${scssOptions.output}${scssOptions.name}`,
        `${scssOptions.output}${scssOptions.sourcemap}${scssOptions.name}.map`,
        `${esOptions.output}**.*js`,
        `${esOptions.output}${esOptions.sourcemap}**.*js.map`
      ], 
      { read: false }
    )
    .pipe(plugins.clean());
});

/// 清除 js、css 发布目录
gulp.task('clean-all', () => {
  console.log('清理目录');
  return gulp
    .src(
      [
        sassWatch.output,
        esWatch.output
      ], 
      { read: false }
    )
    .pipe(plugins.clean());
});

/// 监听文件变化并刷新浏览器
gulp.task('rf', () => {
  browserSync.init({
    server: {
      baseDir: './',
      middleware: [api],
    }
  });
  return gulp
    .watch(syncs)
    .on('change', () => {
      utils.printDate('文件有变化开始刷新');
      browserSync.reload();
    });
});

/// 默认（启动配置中的任务）
gulp.task('default', ['clean-jscss'], e => gulp.start('compile'));

/// 编译任务
gulp.task('compile', ['gulp-es'], e => {
  const list = [];
  rf && list.push('rf');
  produce || list.push('watch-es');
  gulp.start.apply(gulp, list);
});